/**
 * Copyright 2004-2012 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package JCSP.Encryption;

import ru.CryptoPro.JCP.params.CryptParamsInterface;
import ru.CryptoPro.JCP.params.CryptParamsSpec;
import ru.CryptoPro.JCP.tools.Encoder;
import ru.CryptoPro.JCSP.JCSP;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import java.security.SecureRandom;
import java.util.Arrays;

/**
 * Пример шифрования на симметричном ключе на алгоритме 28147
 * с помощью JCSP.
 */
public class SimpleEncryptionExample {

    /**
     * Выполнение шифрования.
     *
     * @param simProvider Провайдер для генерации симметричного ключа.
     * @param cryptParams Параметры шифрования.
     * @throws Exception
     */
    public static void execute(String simProvider, CryptParamsInterface cryptParams)
        throws Exception {

        final String defaultProvider = JCSP.PROVIDER_NAME;
        final String message = "Message for encryption and decryption";
        final SecureRandom rnd =
            SecureRandom.getInstance("CPRandom", defaultProvider);

        byte [] iv = new byte[8];
        rnd.nextBytes(iv);

        IvParameterSpec params = new IvParameterSpec(iv);

        // 1. Генерируем симметричный ключ.

        KeyGenerator kg =
            KeyGenerator.getInstance(JCSP.GOST_CIPHER_NAME, simProvider);

        if (cryptParams != null) {
            kg.init(cryptParams);
        }

        SecretKey symmetricKey = kg.generateKey();

        // 2. Шифруем данные.

        Cipher encrypt = Cipher.getInstance(JCSP.GOST_CIPHER_NAME,
            defaultProvider);
        encrypt.init(Cipher.ENCRYPT_MODE, symmetricKey, params);
        byte[] encryptedMessage = encrypt.doFinal(message.getBytes());

        System.out.println("Source Message: " + message);

        Encoder encoder = new Encoder();
        System.out.println("Encrypted Message: " + encoder.encode(encryptedMessage));

        // 3. Расшифруем данные и проверим.

        Cipher decrypt = Cipher.getInstance(JCSP.GOST_CIPHER_NAME,
            defaultProvider);
        decrypt.init(Cipher.DECRYPT_MODE, symmetricKey, params);
        byte[] decryptedMessage = decrypt.doFinal(encryptedMessage);

        if (message.length() != decryptedMessage.length) {
            throw new Exception("Invalid length of encrypted or decrypted message.");
        } // if

        if (!Arrays.equals(message.getBytes(), decryptedMessage)) {
            throw new Exception("Invalid encrypted or decrypted message.");
        } // if

        System.out.println("Decrypted Message: " +
            new String(decryptedMessage));
    }

    /**
     *
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        main_different_ecrypt(true);
    }

    /**
     * Использование различных параметров для шифрования.
     *
     * @param gost2012Enabled True, если выполнять пример с
     * применением ГОСТ 2012.
     * @throws Exception
     */
    public static void main_different_ecrypt(boolean gost2012Enabled)
        throws Exception {

        // ГОСТ Р 34.10-2001
        execute(JCSP.PROVIDER_NAME, null);

        if (gost2012Enabled) {
            // ГОСТ Р 34.10-2012
            execute(JCSP.PROVIDER_NAME, CryptParamsSpec.getInstance(CryptParamsSpec.Rosstandart_TC26_Z));
        } // if

    }

}
