/**
 * Copyright 2004-2012 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package JTLS_samples.connection;

import ru.CryptoPro.JCP.JCP;
import util.ResolveProvider;

import java.io.File;
import java.util.Arrays;

/**
 * Класс SSL-кофигурации для TLS-примеров.
 */
public class SSLConfiguration {

    /**
     * Тип контейнера JCSP по умолчанию.
     */
    // TODO: добавить проверку if windows then registry else hdimage.
    private static final String JCSP_DEFAULT_STORE_TYPE = ResolveProvider.ALTERNATIVE_HD_IMAGE;
    /**
     * Пароль для всех по умолчанию.
     */
    public static final char[] DEFAULT_PASSWORD = "1".toCharArray();
    /**
     * Относительный путь к корневому доверенному хранилищу локального Тестового УЦ.
     */
    public static final String LOCAL_CA_REL_PATH = "local_ca";
    /**
     * Путь к хранилищу доверенного корневого сертификата локального УЦ (test-ca).
     */
    public static final String LOCAl_TEST_CA_STORE = System.getProperty("user.dir") +
        File.separator + "data" + File.separator + "KEYS" + File.separator +
        LOCAL_CA_REL_PATH + File.separator + "truststore.store";

    /**
     * Переменная, сообщающая о задействованном Java CSP.
     */
    protected boolean JCSPEnabledForTls = ResolveProvider.JCSPEnabledForTls;

    /**
     * Тип доверенного хранилища.
     */
    protected String trustStoreType = null;
    /**
     * Путь к доверенному хранилищу.
     */
    protected String trustStore = null;
    /**
     * Пароль для доступа к доверенному хранилищу.
     */
    protected char[] trustStorePassword = null;
    /**
     * True, если требуется двусторонняя аутентификация.
     */
    protected boolean clientAuth = false;
    /**
     * Тип ключевого контейнера.
     */
    protected String keyStoreType = null;
    /**
     * Алиас ключа. Используется JCSP+JTLS.
     */
    protected String keyAlias = null;
    /**
     * Пароль для доступа к ключевому контейнеру.
     */
    protected char[] keyStorePassword = null;

    /**
     * Создание конфигурации по умолчанию для локального или внешнего УЦ.
     *
     * @param JCSPEnabled true, если задействован Java CSP.
     * @param clientAuth True, если используется клиентская аутентификация.
     * @throws Exception
     */
    public SSLConfiguration(boolean JCSPEnabled, boolean clientAuth)
        throws Exception {
        this(JCSPEnabled, JCP.CERT_STORE_NAME, LOCAl_TEST_CA_STORE,
            DEFAULT_PASSWORD, clientAuth, JCP.HD_STORE_NAME, null,
                DEFAULT_PASSWORD);
    }

    /**
     * Создание конфигурации по умолчанию для локального или внешнего УЦ.
     *
     * @param clientAuth True, если используется клиентская аутентификация.
     * @throws Exception
     */
    public SSLConfiguration(boolean clientAuth) throws Exception {
        this(ResolveProvider.JCSPEnabledForTls, clientAuth);
    }

    /**
     * Создание конфигурации согласно пользовательским настройкам.
     *
     * @param config Пользовательские настройки.
     * @throws Exception
     */
    public SSLConfiguration(ClientConfiguration config)
        throws Exception {
        this(ResolveProvider.JCSPEnabledForTls, config);
    }

    /**
     * Создание конфигурации согласно пользовательским настройкам.
     *
     * @param JCSPEnabled true, если задействован Java CSP.
     * @param config Пользовательские настройки.
     * @throws Exception
     */
    public SSLConfiguration(boolean JCSPEnabled, ClientConfiguration
        config) throws Exception {

        this(

            JCSPEnabled,

            config.getTrustStoreType(),
            config.getTrustStore(),

            config.getTrustStorePassword() != null
                ? config.getTrustStorePassword().toCharArray()
                : null,
                config.getUseClientAuth(),

            config.getKeyStoreType(),

            config.getKeyStoreAlias() != null
                ? config.getKeyStoreAlias()
                : null,

            config.getKeyStorePassword() != null
                ? config.getKeyStorePassword().toCharArray()
                : null

        );
    }

    /**
     * Создание конфигурации.
     *
     * @param JCSPEnabled true, если задействован Java CSP.
     * @param trustStoreType Тип доверенного хранилища.
     * @param trustStore Путь к доверенному хранилищу.
     * @param trustStorePassword Пароль для доступа к доверенному хранилищу.
     * @param clientAuth True, если используется клиентская аутентификация.
     * @param keyStoreType Тип ключевого контейнера.
     * @param keyStoreAlias Алиас ключа (для JCSP). Может быть null.
     * @param keyStorePassword Пароль для доступа к ключевому контейнеру.
     * @throws Exception
     *
     */
    public SSLConfiguration(boolean JCSPEnabled, String trustStoreType,
        String trustStore, char[] trustStorePassword, boolean clientAuth,
        String keyStoreType, String keyStoreAlias, char[] keyStorePassword)
        throws Exception {

        JCSPEnabledForTls = JCSPEnabled;
        this.keyStoreType = JCSPEnabledForTls
                ? ((keyStoreType == null) ? JCSP_DEFAULT_STORE_TYPE : keyStoreType)
                : ((keyStoreType == null) ? JCP.HD_STORE_NAME       : keyStoreType);

        this.keyAlias = keyStoreAlias;
        this.keyStorePassword = keyStorePassword;
        this.clientAuth = clientAuth;

        this.trustStoreType = trustStoreType == null ? JCP.CERT_STORE_NAME : trustStoreType;

        if (trustStore == null || trustStore.length() == 0) {
            throw new Exception("Trust store is null or empty.");
        } // if

        this.trustStore = trustStore;
        this.trustStorePassword = trustStorePassword;
    }

    /**
     * Создание конфигурации.
     *
     * @param trustStoreType Тип доверенного хранилища.
     * @param trustStore Путь к доверенному хранилищу.
     * @param trustStorePassword Пароль для доступа к доверенному хранилищу.
     * @param clientAuth True, если используется клиентская аутентификация.
     * @param keyStoreType Тип ключевого контейнера.
     * @param keyStoreAlias Алиас ключа (для JCSP). Может быть null.
     * @param keyStorePassword Пароль для доступа к ключевому контейнеру.
     * @throws Exception
     *
     */
    public SSLConfiguration(String trustStoreType, String trustStore,
         char[] trustStorePassword, boolean clientAuth, String keyStoreType,
         String keyStoreAlias, char[] keyStorePassword) throws Exception {

        this(ResolveProvider.JCSPEnabledForTls, trustStoreType, trustStore,
            trustStorePassword, clientAuth, keyStoreType, keyStoreAlias,
                keyStorePassword);

    }

    /**
     * Позволяет узнать, используется ли провайдер JCSP.
     *
     * @return true, если используется.
     */
    public boolean isJCSPEnabledForTls() {
        return JCSPEnabledForTls;
    }

    /**
     * Позволяет узнать тип доверенного хранилища сертификатов.
     *
     * @return тип хранилища.
     */
    public String getTrustStoreType() {
        return trustStoreType;
    }

    /**
     * Позволяет узнать путь к доверенному хранилищу сертификатов.
     *
     * @return путь к хранилищу.
     */
    public String getTrustStore() {
        return trustStore;
    }

    /**
     * Позволяет узнать пароль к доверенному хранилищу сертификатов.
     *
     * @return пароль к хранилищу.
     */
    public char[] getTrustStorePassword() {
        return trustStorePassword;
    }

    /**
     * Позволяет узнать пароль к доверенному хранилищу сертификатов в виде строки.
     *
     * @return пароль к хранилищу.
     */
    public String getTrustStorePasswordString() {
        return trustStorePassword != null ? String.copyValueOf(trustStorePassword) : "";
    }

    /**
     * Позволяет узнать, требуется ли аутентификация клиента.
     *
     * @return True, если требуется.
     */
    public boolean needClientAuth() {
        return clientAuth;
    }

    /**
     * Позволяет узнать тип ключевого контейнера.
     *
     * @return тип контейнера.
     */
    public String getKeyStoreType() {
        return keyStoreType;
    }

    /**
     * Позволяет получить алиас ключа.
     *
     * @return алиас ключа или null.
     */
    public String getKeyAlias() {
        return keyAlias;
    }

    /**
     * Позволяет узнать пароль к ключевому контейнеру.
     *
     * @return пароль к контейнеру.
     */
    public char[] getKeyStorePassword() {
        return keyStorePassword;
    }

    /**
     * Позволяет узнать пароль к ключевому контейнеру в виде строки.
     *
     * @return пароль к контейнеру.
     */
    public String getKeyStorePasswordString() {
        return keyStorePassword != null ? String.copyValueOf(keyStorePassword) : "";
    }

    /**
     * Сравнение конфигураций.
     *
     * @param configuration Сравниваемая конфигурация.
     * @return True, если конфигурации идентичные.
     */
    public boolean equals(SSLConfiguration configuration) {

        boolean equalKeyAliases =
            (!(keyAlias != null && configuration.getKeyAlias() != null)) ||
            keyAlias.equalsIgnoreCase(configuration.getKeyAlias());

        return JCSPEnabledForTls == configuration.isJCSPEnabledForTls()
            && trustStoreType.equalsIgnoreCase(configuration.trustStoreType)
            && trustStore.equalsIgnoreCase(configuration.trustStore)
            && Arrays.equals(trustStorePassword, configuration.trustStorePassword)
            && keyStoreType.equalsIgnoreCase(configuration.keyStoreType)
            && equalKeyAliases
            && Arrays.equals(keyStorePassword, configuration.keyStorePassword)
            && (clientAuth || clientAuth == configuration.clientAuth);
    }

}
