/**
 * Copyright 2004-2013 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp;

import android.app.ActionBar;
import android.app.FragmentTransaction;
import android.os.Bundle;

import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;

import android.util.Log;

import android.widget.EditText;
import android.widget.TextView;

import ru.CryptoPro.ACSPClientApp.client.example.base.EncryptDecryptData;
import ru.CryptoPro.ACSPClientApp.util.*;

import ru.CryptoPro.AdES.AdESConfig;
import ru.CryptoPro.JCPxml.XmlInit;

import ru.CryptoPro.JCSP.CSPConfig;
import ru.CryptoPro.JCSP.JCSP;
import ru.CryptoPro.JCSP.support.BKSTrustStore;

import ru.CryptoPro.reprov.RevCheck;
import ru.CryptoPro.ssl.android.util.cpSSLConfig;

import ru.cprocsp.ACSP.tools.common.CSPTool;
import ru.cprocsp.ACSP.tools.common.Constants;
import ru.cprocsp.ACSP.tools.common.Infrastructure;
import ru.cprocsp.ACSP.tools.common.RawResource;
import ru.cprocsp.ACSP.tools.common.AppUtils;

import java.io.File;
import java.security.Security;
import java.util.*;

/**
 * Главная activity приложения. Выполнение инициализации CSP,
 * создание вкладок с примерами, копирования тестовых контейнеров
 * и др.
 *
 */
public class MainActivity extends FragmentActivity implements
    ActionBar.TabListener {

    /**
     * Элемент для отображения вкладки.
     */
    private ViewPager viewPager = null;

    /**
     * Called when the activity is first created.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        Log.i(Constants.APP_LOGGER_TAG, "Load application: " +
            getPackageName());

        // Версия приложения в заголовке.

        String title = getString(R.string.app_name) +
            ", v. " + AppUtils.getApplicationVersion(this);

        setTitle(title);

        // 1. Инициализация провайдеров: CSP и java-провайдеров
        // (Обязательная часть).

        if (!initCSPProviders()) {
            Log.i(Constants.APP_LOGGER_TAG, "Couldn't initialize CSP.");
            return;
        } // if

        initJavaProviders();

        // 2. Управление вкладками и панелями.

        initNavigation();

        // 3. Копирование тестовых контейнеров для подписи,
        // проверки подписи, шифрования и TLS (Примеры и вывод
        // в лог).

        initLogger();
        installContainers();

        // 4. Инициируем объект для управления выбором типа
        // контейнера (Настройки).

        KeyStoreType.init(this);

        // 5. Инициируем объект для управления выбором типа
        // провайдера (Настройки).

        ProviderType.init(this);

        // 6. Вывод информации о тестовых контейнерах.

        logTestContainers();

        // 7. Вывод информации о провайдере и контейнерах
        // (Пример).

        logJCspServices();

        // Для логирования: CSPConfig.setNeedLogBioStatistics(true);

    }

    @Override
    public void onTabUnselected(ActionBar.Tab tab,
        FragmentTransaction fragmentTransaction) {
        ;
    }

    @Override
    public void onTabSelected(ActionBar.Tab tab,
        FragmentTransaction fragmentTransaction) {
        viewPager.setCurrentItem(tab.getPosition());
    }

    @Override
    public void onTabReselected(ActionBar.Tab tab,
        FragmentTransaction fragmentTransaction) {
        ;
    }

    /**
     * Создание вкладок и меню.
     *
     */
    private void initNavigation() {

        final FragmentManager fragmentManager =
            getSupportFragmentManager();

        SectionsPagerAdapter sectionsPagerAdapter =
            new SectionsPagerAdapter(fragmentManager);

        final ActionBar actionBar = getActionBar();
        actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);

        viewPager = (ViewPager) findViewById(R.id.pager);
        viewPager.setAdapter(sectionsPagerAdapter);
        viewPager.setOffscreenPageLimit(2);
        viewPager.setVerticalScrollBarEnabled(true);

        // Смена вкладок.
        viewPager.setOnPageChangeListener(
            new ViewPager.SimpleOnPageChangeListener() {

                @Override
                public void onPageSelected(int position) {

                    actionBar.setSelectedNavigationItem(position);

                    String tag = AppUtils.makeFragmentTag(viewPager, position);
                    Fragment fragment = fragmentManager.findFragmentByTag(tag);

                    if (fragment instanceof Observer) {
                        ((Observer) fragment).update(null, null);
                    } // if

                }

            });

        // Заголовки вкладок.
        for (int i = 0; i < sectionsPagerAdapter.getCount(); i++) {
            actionBar.addTab(actionBar.newTab()
                .setText(sectionsPagerAdapter.getPageTitle(i))
                .setTabListener(this));
        } // for

    }

    /************************ Инициализация провайдера ************************/

    /**
     * Инициализация CSP провайдера.
     *
     * @return True в случае успешной инициализации.
     */
    private boolean initCSPProviders() {

        // Инициализация провайдера CSP. Должна выполняться
        // один раз в главном потоке приложения, т.к. использует
        // статические переменные.
        //
        // 1. Создаем инфраструктуру CSP и копируем ресурсы
        // в папку. В случае ошибки мы, например, выводим окошко
        // (или как-то иначе сообщаем) и завершаем работу.

        int initCode = CSPConfig.init(this);
        boolean initOk = initCode == CSPConfig.CSP_INIT_OK;

        // Если инициализация не удалась, то сообщим об ошибке.
        if (!initOk) {

            switch (initCode) {

                // Не передан контекст приложения (null). Он необходим,
                // чтобы произвести копирование ресурсов CSP, создание
                // папок, смену директории CSP и т.п.
                case CSPConfig.CSP_INIT_CONTEXT:
                    AppUtils.errorMessage(this, "Couldn't initialize context.");
                break;

                /**
                 * Не удается создать инфраструктуру CSP (папки): нет
                 * прав (нарушен контроль целостности) или ошибки.
                 * Подробности в logcat.
                 */
                case CSPConfig.CSP_INIT_CREATE_INFRASTRUCTURE:
                    AppUtils.errorMessage(this, "Couldn't create CSP infrastructure.");
                break;

                /**
                 * Не удается скопировать все или часть ресурсов CSP -
                 * конфигурацию, лицензию (папки): нет прав (нарушен
                 * контроль целостности) или ошибки.
                 * Подробности в logcat.
                 */
                case CSPConfig.CSP_INIT_COPY_RESOURCES:
                    AppUtils.errorMessage(this, "Couldn't copy CSP resources.");
                break;

                /**
                 * Не удается задать рабочую директорию для загрузки
                 * CSP. Подробности в logcat.
                 */
                case CSPConfig.CSP_INIT_CHANGE_WORK_DIR:
                    AppUtils.errorMessage(this, "Couldn't change CSP working directory.");
                break;

                /**
                 * Неправильная лицензия.
                 */
                case CSPConfig.CSP_INIT_INVALID_LICENSE:
                    AppUtils.errorMessage(this, "Invalid CSP serial number.");
                break;

                /**
                 * Не удается создать хранилище доверенных сертификатов
                 * для CAdES API.
                 */
                case CSPConfig.CSP_TRUST_STORE_FAILED:
                    AppUtils.errorMessage(this, "Couldn't create trust store for CAdES API.");
                    break;

                /**
                 * Не удается сохранить путь к библиотекам провайдера
                 * в конфиг.
                 */
                case CSPConfig.CSP_STORE_LIBRARY_PATH:
                    AppUtils.errorMessage(this, "Couldn't store native library path to config.");
                    break;

                /**
                 * Ошибка контроля целостности.
                 */
                case CSPConfig.CSP_INIT_INVALID_INTEGRITY:
                    AppUtils.errorMessage(this, "Integrity control failure.");
                    break;

            } // switch

        } // if

        return initOk;
    }

    @Override
    public void onResume() {

        super.onResume();

        // Необходимо для отображения диалоговых окон
        // ДСЧ, ввода пин-кода и сообщений.

        CSPConfig.registerActivityContext(this);

    }

    /**
     * Добавление нативного провайдера Java CSP,
     * SSL-провайдера и Revocation-провайдера в
     * список Security. Инициализируется JCPxml,
     * CAdES.
     *
     * Происходит один раз при инициализации.
     * Возможно только после инициализации в CSPConfig!
     *
     */
    private void initJavaProviders() {

        // Загрузка Java CSP (хеш, подпись, шифрование,
        // генерация контейнеров).

        if (Security.getProvider(JCSP.PROVIDER_NAME) == null) {
            Security.addProvider(new JCSP());
        } // if

        // Загрузка Java TLS (TLS).

        // Необходимо переопределить свойства, чтобы использовались
        // менеджеры из cpSSL, а не Harmony.

        Security.setProperty("ssl.KeyManagerFactory.algorithm",
            ru.CryptoPro.ssl.android.Provider.KEYMANGER_ALG);
        Security.setProperty("ssl.TrustManagerFactory.algorithm",
            ru.CryptoPro.ssl.android.Provider.KEYMANGER_ALG);

        Security.setProperty("ssl.SocketFactory.provider",
            "ru.CryptoPro.ssl.android.SSLSocketFactoryImpl");
        Security.setProperty("ssl.ServerSocketFactory.provider",
            "ru.CryptoPro.ssl.android.SSLServerSocketFactoryImpl");

        if (Security.getProvider(ru.CryptoPro.ssl.android.Provider.PROVIDER_NAME) == null) {
            Security.addProvider(new ru.CryptoPro.ssl.android.Provider());
        } // if

        // Провайдер хеширования, подписи, шифрования по умолчанию.
        cpSSLConfig.setDefaultSSLProvider(JCSP.PROVIDER_NAME);

        // Загрузка Revocation Provider (CRL, OCSP).

        if (Security.getProvider(RevCheck.PROVIDER_NAME) == null) {
            Security.addProvider(new RevCheck());
        } // if

        // Инициализация XML DSig (хеш, подпись).

        XmlInit.init();

        // %%% Параметры для Java TLS и CAdES API %%%

        // Провайдер CAdES API по умолчанию.
        AdESConfig.setDefaultProvider(JCSP.PROVIDER_NAME);

        // Отключаем проверку цепочки штампа времени (CAdES-T),
        // чтобы не требовать него CRL.

        System.setProperty("ru.CryptoPro.CAdES.validate_tsp", "false");

        // Включаем возможность онлайновой проверки статуса
        // сертификата.

        System.setProperty("com.sun.security.enableCRLDP", "true");
        System.setProperty("com.ibm.security.enableCRLDP", "true");

        // Настройки TLS для генерации контейнера и выпуска сертификата
        // в УЦ 2.0, т.к. обращение к УЦ 2.0 будет выполняться по протоколу
        // HTTPS и потребуется авторизация по сертификату. Указываем тип
        // хранилища с доверенным корневым сертификатом, путь к нему и пароль.

        final String trustStorePath = getApplicationInfo().dataDir + File.separator +
            BKSTrustStore.STORAGE_DIRECTORY + File.separator + BKSTrustStore.STORAGE_FILE_TRUST;

        final String trustStorePassword = String.valueOf(BKSTrustStore.STORAGE_PASSWORD);
        Log.d(Constants.APP_LOGGER_TAG, "Default trust store: " + trustStorePath);

        System.setProperty("javax.net.ssl.trustStoreType", BKSTrustStore.STORAGE_TYPE);
        System.setProperty("javax.net.ssl.trustStore", trustStorePath);
        System.setProperty("javax.net.ssl.trustStorePassword", trustStorePassword);

    }

    /************************ Поле для вывода логов *************************/

    /**
     * Инициализация объекта для отображения логов.
     *
     */
    private void initLogger() {

        // Поле для вывода логов и метка для отображения
        // статуса.

        EditText etLog = (EditText) findViewById(R.id.etLog);
        etLog.setMinLines(10);

        TextView tvOpStatus = (TextView) findViewById(R.id.tvOpStatus);

        Logger.init(getResources(), etLog, tvOpStatus);
        Logger.clear();

    }

    /******************************** Адаптер для фрагментов ********************************/

    /**
     * Класс SectionsPagerAdapter предназначен для управления
     * вкладками и их активации.
     *
     */
    public class SectionsPagerAdapter extends FragmentPagerAdapter {

        /**
         * Конструктор.
         *
         * @param fm Менеджер фрагментов.
         */
        public SectionsPagerAdapter(FragmentManager fm) {
            super(fm);
        }

        @Override
        public Fragment getItem(int i) {

            switch (i) {

                case 0: return new ContainerActivity();
                case 1: return new ExamplesActivity();
                case 2: return new ACSPIntentActivity();
                case 3: return new UtilActivity();
                case 4: return new SettingsActivity();

            } // switch

            return null;

        }

        @Override
        public int getCount() {
            return 5;
        }

        @Override
        public CharSequence getPageTitle(int position) {

            switch (position) {

                case 0: return getString(R.string.ContainerTab).toUpperCase();
                case 1: return getString(R.string.ExamplesTab).toUpperCase();
                case 2: return getString(R.string.IntentsTab).toUpperCase();
                case 3: return getString(R.string.UtilitiesTab).toUpperCase();
                case 4: return getString(R.string.SettingsTab).toUpperCase();

            } // switch

            return "";

        }

    }

    /************************** Служебные функции ****************************/

    /**
     * Вывод списка поддерживаемых алгоритмов.
     *
     */
    private void logJCspServices() {
        ProviderServiceInfo.logServiceInfo(KeyStoreUtil.DEFAULT_PROVIDER);
    }

    /**
     * Информация о тестовых контейнерах.
     *
     */
    private void logTestContainers() {

        // Список алиасов контейнеров.
        final String[] aliases = {
            EncryptDecryptData.CLIENT_CONTAINER_NAME,          // ГОСТ 34.10-2001
            EncryptDecryptData.SERVER_CONTAINER_NAME,          // ГОСТ 34.10-2001
            EncryptDecryptData.CLIENT_CONTAINER_2012_256_NAME, // ГОСТ 34.10-2012 (256)
            EncryptDecryptData.SERVER_CONTAINER_2012_256_NAME, // ГОСТ 34.10-2012 (256)
            EncryptDecryptData.CLIENT_CONTAINER_2012_512_NAME, // ГОСТ 34.10-2012 (512)
            EncryptDecryptData.SERVER_CONTAINER_2012_512_NAME  // ГОСТ 34.10-2012 (512)
        };

        // Список паролей контейнеров.
        final char[][] passwords = {
            EncryptDecryptData.CLIENT_KEY_PASSWORD,
            EncryptDecryptData.SERVER_KEY_PASSWORD,
            EncryptDecryptData.CLIENT_KEY_2012_256_PASSWORD,
            EncryptDecryptData.CLIENT_KEY_2012_256_PASSWORD,
            EncryptDecryptData.CLIENT_KEY_2012_512_PASSWORD,
            EncryptDecryptData.CLIENT_KEY_2012_512_PASSWORD
        };

        final String format = getString(R.string.ContainerAboutTestContainer);
        Logger.log("$$$ About test containers $$$");

        for (int i = 0; i < aliases.length; i++) {
            final String aboutTestContainer = String.format(format, aliases[i], String.valueOf(passwords[i]));
            Logger.log("** " + i + ") " + aboutTestContainer);
        } // for

    }

    /**
     * Копирование тестовых контейнеров для подписи, шифрования,
     * обмена по TLS в папку keys.
     *
     */
    private void installContainers() {

        // Имена файлов в контейнере.
        final String[] pseudos = {
                "header.key",
                "masks.key",
                "masks2.key",
                "name.key",
                "primary.key",
                "primary2.key"
        };

        // Список алиасов контейнеров.
        final String[] aliases = {
                EncryptDecryptData.CLIENT_CONTAINER_NAME,          // ГОСТ 34.10-2001
                EncryptDecryptData.SERVER_CONTAINER_NAME,          // ГОСТ 34.10-2001
                EncryptDecryptData.CLIENT_CONTAINER_2012_256_NAME, // ГОСТ 34.10-2012 (256)
                EncryptDecryptData.SERVER_CONTAINER_2012_256_NAME, // ГОСТ 34.10-2012 (256)
                EncryptDecryptData.CLIENT_CONTAINER_2012_512_NAME, // ГОСТ 34.10-2012 (512)
                EncryptDecryptData.SERVER_CONTAINER_2012_512_NAME  // ГОСТ 34.10-2012 (512)
        };

        // Список контейнеров и файлов внутри.
        final Integer[][] containers = {
                {R.raw.clienttls_header, R.raw.clienttls_masks, R.raw.clienttls_masks2, R.raw.clienttls_name, R.raw.clienttls_primary, R.raw.clienttls_primary2},
                {R.raw.servertls_header, R.raw.servertls_masks, R.raw.servertls_masks2, R.raw.servertls_name, R.raw.servertls_primary, R.raw.servertls_primary2},
                {R.raw.cli12256_header,  R.raw.cli12256_masks,  R.raw.cli12256_masks2,  R.raw.cli12256_name,  R.raw.cli12256_primary,  R.raw.cli12256_primary2 },
                {R.raw.ser12256_header,  R.raw.ser12256_masks,  R.raw.ser12256_masks2,  R.raw.ser12256_name,  R.raw.ser12256_primary,  R.raw.ser12256_primary2 },
                {R.raw.cli12512_header,  R.raw.cli12512_masks,  R.raw.cli12512_masks2,  R.raw.cli12512_name,  R.raw.cli12512_primary,  R.raw.cli12512_primary2 },
                {R.raw.ser12512_header,  R.raw.ser12512_masks,  R.raw.ser12512_masks2,  R.raw.ser12512_name,  R.raw.ser12512_primary,  R.raw.ser12512_primary2 }
        };

        // Копирование контейнеров.

        try {

            for (int i = 0; i < containers.length; i++) {

                final Integer[] container = containers[i];
                final Map<Integer, String> containerFiles = new HashMap<Integer, String>();

                for (int j = 0; j < container.length; j++) {
                    containerFiles.put(container[j], pseudos[j]);
                } // for

                installContainer(aliases[i], containerFiles);

            } // for

        } catch (Exception e) {
            Log.e(Constants.APP_LOGGER_TAG, e.getMessage(), e);
        }

    }

    /**
     * Копирование файлов контейнера в папку согласно названию
     * контейнера.
     *
     * @param containerName Имя папки контейнера.
     * @param containerFiles Список и ссылки на файлы контейнера.
     * @throws Exception
     */
    private void installContainer(String containerName,
        Map<Integer, String> containerFiles) throws Exception {

        String resourceDirectory = Infrastructure.userName2Dir(this)
            + File.separator + containerName;

        Log.i(Constants.APP_LOGGER_TAG, "Install container: " +
            containerName + " to resource directory: " + resourceDirectory);

        CSPTool cspTool = new CSPTool(this);

        // Копируем ресурсы  в папку keys.

        RawResource resource = cspTool.createRawResource(
            Constants.CSP_SOURCE_TYPE_CONTAINER, resourceDirectory);

        Iterator<Integer> iterator = containerFiles.keySet().iterator();

        while (iterator.hasNext()) {

            Integer index = iterator.next();
            String fileName = containerFiles.get(index);

            if (!resource.copy(index, fileName, true)) {
                throw new Exception("Couldn't copy " + fileName);
            } // if

        } // while
    }

}
