/**
 * $RCSfileInstallCAdESTestTrustCertExample.java,v $
 * version $Revision: 36379 $
 * created 15.09.2014 10:21 by Yevgeniy
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 *
 * Copyright 2004-2014 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import ru.CryptoPro.ACSPClientApp.R;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.CAdESData;
import ru.CryptoPro.ACSPClientApp.util.Logger;
import ru.CryptoPro.JCSP.CSPConfig;
import ru.CryptoPro.JCSP.support.BKSTrustStore;

import java.io.*;
import java.security.KeyStore;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

/**
 * Класс InstallCAdESTestTrustCertExample реализует пример
 * добавления тестового корневого сертификата в специальное
 * хранилище доверенных сертификатов, которое используется в
 * CAdES API и создается один раз в JInitCSP.init(). В данное
 * хранилище следует помещать те корневые сертификаты, которые
 * будут и должны использоваться при построении цепочек сертификатов
 * в CAdES API по аналогии с cacerts в SUN/IBM JRE.
 * Помимо этого хранилища, в CAdES API используется хранилище
 * доверенных сертификатов AndroidCAStore для загрузки корневых
 * сертификатов, установка сертификатов в которое происходит с помощью
 * "Настройки"->"Безопасность"->"Установить с карты памяти" в
 * Android >= 4).
 *
 * @author Copyright 2004-2014 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class InstallCAdESTestTrustCertExample implements CAdESData {

    /**
     * Пароль к хранилищу доверенных сертификатов по умолчанию.
     */
    private static final char[] DEFAULT_TRUST_STORE_PASSWORD = BKSTrustStore.STORAGE_PASSWORD;

    /**
     * Устанавливаемые корневые сертификаты.
     */
    private List<X509Certificate> trustCerts = new ArrayList<X509Certificate>(2);

    /**
     * Путь к хранилищу доверенных сертификатов для установки сертификатов.
     */
    private String trustStore = null;

    /**
     * Файл хранилища.
     */
    private File trustStoreFile = null;

    /**
     * Конструктор. Подготовка списка корневых сертификатов для установки.
     *
     * @param adapter Настройки примера.
     * @throws Exception
     */
    public InstallCAdESTestTrustCertExample(ContainerAdapter adapter) throws Exception {

        InputStream trustStreamForSigner = null, trustStreamForTsp = null;
        trustStore = CSPConfig.getBksTrustStore() + File.separator +
            BKSTrustStore.STORAGE_FILE_TRUST;

        checkTrustStore();

        // Пока все клиентские сертификаты под одним корневым.
        switch (adapter.getProviderType()) {
            case pt2012Short:
            case pt2012Long:
            default:
                trustStreamForSigner = adapter.getResources().openRawResource(R.raw.ext_test_ca);  // root certificate for signer
                trustStreamForTsp    = adapter.getResources().openRawResource(R.raw.ext_tsp_root); // root certificate for tsp
        } // switch

        loadCert(trustStreamForSigner);
        loadCert(trustStreamForTsp);

    }

    /**
     * Загрузка сертификата из потока в список.
     *
     * @param trustStream Поток данных.
     * @throws Exception
     */
    private void loadCert(InputStream trustStream) throws  Exception {

        try {

            final CertificateFactory factory = CertificateFactory.getInstance("X.509");
            trustCerts.add((X509Certificate) factory.generateCertificate(trustStream));

        } finally {

            if (trustStream != null) {

                try {
                    trustStream.close();
                } catch (IOException e) {
                    ;
                }

            } // if

        }

    }

    @Override
    public void getResult() throws Exception {

        Logger.log("Load trust store '" + trustStore + "'");

        int  i = 0;
        for (X509Certificate trustCert : trustCerts) {
            saveTrustCert(trustStoreFile, trustCert, i == 0);
            i++;
        } // for

        Logger.setStatusOK();

    }

    /**
     * Проверка существования хранилища.
     *
     * @throws Exception
     */
    private void checkTrustStore() throws Exception {

        trustStoreFile = new File(trustStore);
        if (!trustStoreFile.exists()) {
            throw new Exception("Trust store " + trustStore +
                " doesn't exist");
        } // if

    }

    /**
     * Вывод списка сертификатов в хранилище.
     *
     * @param keyStore Хранилище.
     * @throws Exception
     */
    private void printAliases(KeyStore keyStore) throws Exception {

        Enumeration<String> aliases = keyStore.aliases();
        int i = 0;

        while (aliases.hasMoreElements()) {

            i++;
            Logger.log("** # " + i);

            String alias = aliases.nextElement();
            Logger.log("* Alias: " + alias);

            X509Certificate cert = (X509Certificate) keyStore.getCertificate(alias);
            Logger.log("* Certificate sn: " +
                cert.getSerialNumber().toString(16) +
                ", subject: " + cert.getSubjectDN());

        } // while

    }

    /**
     * Сохранение сертификата в хранилище.
     *
     * @param trustStoreFile Файл хранилища.
     * @param trustCert Корневой сертификат, добавляемый в хранилище.
     * @param needPrintAliases True, если выводить содержимое хранилища.
     * @throws Exception
     */
    private void saveTrustCert(File trustStoreFile, X509Certificate
        trustCert, boolean needPrintAliases) throws Exception {

        FileInputStream storeStream = new FileInputStream(trustStore);
        KeyStore keyStore = KeyStore.getInstance(BKSTrustStore.STORAGE_TYPE);

        keyStore.load(storeStream, DEFAULT_TRUST_STORE_PASSWORD);
        storeStream.close();

        Logger.log("Certificate sn: " +
            trustCert.getSerialNumber().toString(16) +
            ", subject: " + trustCert.getSubjectDN());

        // Будущий алиас корневого сертификата в хранилище.
        String trustCertAlias = trustCert.getSerialNumber().toString(16);

        // Вывод списка содержащищся в хранилище сертификатов.
        Logger.log("Current count of trusted certificates: " + keyStore.size());

        if (needPrintAliases) {
            printAliases(keyStore);
        } // if

        // Добавление сертификата, если его нет.

        boolean needAdd = (keyStore.getCertificateAlias(trustCert) == null);
        if (needAdd) {

            Logger.log("** Adding the trusted certificate " +
                trustCert.getSubjectDN() + " with alias '" +
                trustCertAlias + "' into the trust store");

            keyStore.setCertificateEntry(trustCertAlias, trustCert);

            FileOutputStream updatedTrustStore = new FileOutputStream(trustStoreFile);
            keyStore.store(updatedTrustStore, DEFAULT_TRUST_STORE_PASSWORD);

            Logger.log("The trusted certificate was added successfully.");

        } // if
        else {
            Logger.log("** Trusted certificate has already " +
                "existed in the trust store.");
        } // else

    }

    @Override
    public boolean isAlreadyInstalled() throws Exception {

        FileInputStream storeStream = new FileInputStream(trustStore);
        KeyStore keyStore = KeyStore.getInstance(BKSTrustStore.STORAGE_TYPE);

        keyStore.load(storeStream, DEFAULT_TRUST_STORE_PASSWORD);
        storeStream.close();

        // Если нет какого-то из сертификатов, то считается, что
        // они не установлены.
        for (X509Certificate crt : trustCerts) {
            if (keyStore.getCertificateAlias(crt) == null) {
                return false;
            } // if
        } // for

        return true;
    }
}
