/**
 * $RCSfileClientConnection.java,v $
 * version $Revision: 36379 $
 * created 16.08.2016 10:09 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2016 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package tls_proxy;

import tls_proxy.event.OnConnectionListener;
import javax.net.ssl.SSLSocket;

import java.io.*;
import java.net.Socket;

import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * Класс ClientConnection предназначен для
 * обслуживания клиентского подключения и
 * передачи данных между сокетами.
 *
 * @author Copyright 2004-2016 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class ClientConnection extends Thread {

    /**
     * Сокет.
     */
    private final Socket socket;

    /**
     * Защищенный сокет.
     */
    private final SSLSocket secureSocket;

    /**
     * Входящий поток локального сокета.
     */
    private final InputStream localSocketInputStream;

    /**
     * Исходящий поток локального сокета.
     */
    private final OutputStream localSocketOutputStream;

    /**
     * Входящий поток защищенного сокета.
     */
    private final InputStream remoteSocketInputStream;

    /**
     * Исходящий поток защищенного сокета.
     */
    private final OutputStream remoteSocketOutputStream;

    /**
     * Последняя дата активности.
     */
    private long lastActivityTime = Calendar.getInstance().getTimeInMillis();

    /**
     * Синхронизация lastActivityTime.
     */
    private final Object lastActivityTimeLock = new Object();

    /**
     * Слушатель событий.
     */
    private OnConnectionListener onConnectionListener;

    /**
     * Флаг завершения работы.
     */
    private boolean closed = false;

    /**
     * Служебный класс для перенеправления данных.
     *
     */
    class StreamCallable implements Callable<Void> {

        /**
         * Входящий поток.
         */
        private final InputStream inputStream;

        /**
         * Исхдящий поток.
         */
        private final OutputStream outputStream;

        /**
         * Конструктор.
         *
         * @param inputSocket Входящий сокет.
         * @param outputSocket Исходящий сокет.
         * @throws Exception
         */
        public StreamCallable(Socket inputSocket,
            Socket outputSocket) throws Exception {

            this.inputStream  = inputSocket.getInputStream();
            this.outputStream = outputSocket.getOutputStream();

        }

        @Override
        public Void call() throws Exception {

            try {

                boolean firstAccess = true;
                byte[] buffer = new byte[16 * 1024 * 1024]; // 16 Kb

                while (!Thread.interrupted()) {

                    long opTime = Calendar.getInstance().getTimeInMillis();
                    int readBufferSize;

                    while ((readBufferSize = inputStream.read(buffer)) != -1) {

                        outputStream.write(buffer, 0, readBufferSize);
                        outputStream.flush();

                        // Чтобы не блокировать слишком часто,
                        // используем время между текущим моментом
                        // и временем последнего действия opTime.
                        // В первый раз тоже запишем время.

                        long diff = (Calendar.getInstance().getTimeInMillis() - opTime);

                        if (firstAccess || diff > 10 * 1000) {
                            updateLastActivityTime();
                        } // if

                        // Фиксируем время действия.

                        opTime = Calendar.getInstance().getTimeInMillis();
                        firstAccess = false;

                    } // if

                    Thread.sleep(500);

                } // while

            } catch (Exception e) {
                MainLogger.error(TAG() + "stream failed", e);
            } finally {

                if (inputStream != null) {

                    try {
                        inputStream.close();
                    } catch (Exception e) {
                        // ignore
                    }

                } // if

                if (outputStream != null) {

                    try {
                        outputStream.close();
                    } catch (Exception e) {
                        // ignore
                    }

                } // if

            }

            MainLogger.info(TAG() + "stream completed!");
            return null;

        }

        /**
         * Обновление даты последнего действия.
         *
         */
        private void updateLastActivityTime() {

            synchronized (lastActivityTimeLock) {
                lastActivityTime = Calendar.getInstance().getTimeInMillis();
            } // synchronized

        }

    }

    /**
     * Связь локальный сокет -> удаленный сокет.
     */
    private final StreamCallable local2remote;

    /**
     * Связь локальный удаленный сокет -> сокет.
     */
    private final StreamCallable remote2local;

    /**
     * Пул потоков.
     */
    private ExecutorService service;

    /**
     * Идентификатор клиента.
     */
    private final int clientId;

    /**
     * Конструктор.
     *
     * @param id Идентификатор клиента.
     * @param socket Сокет.
     * @param secureSocket Защищенный сокет.
     * @throws Exception
     */
    public ClientConnection(int id, Socket socket,
        SSLSocket secureSocket) throws Exception {

        this.clientId = id;
        this.socket = socket;
        this.secureSocket = secureSocket;

        localSocketInputStream  = socket.getInputStream();
        localSocketOutputStream = socket.getOutputStream();

        remoteSocketInputStream  = secureSocket.getInputStream();
        remoteSocketOutputStream = secureSocket.getOutputStream();

        local2remote = new StreamCallable(socket, secureSocket);
        remote2local = new StreamCallable(secureSocket, socket);

    }

    /**
     * Задание слушателя событий для передачи
     * событий управляющему потоку.
     *
     * @param listener Слушатель событий.
     */
    public void setOnConnectionListener(OnConnectionListener listener) {
        onConnectionListener = listener;
    }

    @Override
    public void run() {

        MainLogger.info(TAG() + "client executor started.");
        service = Executors.newFixedThreadPool(2);

        try {

            if (onConnectionListener != null) {
                onConnectionListener.onClientStart(clientId);
            } // if

            service.invokeAll(Arrays.asList(local2remote, remote2local));

        } catch (Exception e) {
            MainLogger.error(TAG() + "executor failed", e);
        } finally {
            close();
        }

        MainLogger.info(TAG() + "client executor completed!");

    }


    /**
     * Проверка активности. Связь считается неактивной,
     * если один из сокетов закрыт или исчерпан таймаут.
     *
     * @return true, если соединение неактивно.
     */
    public boolean isInactive() {

        long tmpLastActivityTime;

        synchronized (lastActivityTimeLock) {
            tmpLastActivityTime = lastActivityTime;
        } // synchronized

        long diff = (Calendar.getInstance().getTimeInMillis() - tmpLastActivityTime);
        long limit = ConfigReader.INSTANCE.getInactiveTimeout();
        boolean inactive = (diff >= limit);

        MainLogger.info(TAG() + "client inactive: " + inactive +
            " (" + diff + " ms, limit " + limit + " ms)");

        return inactive || socket.isClosed() || secureSocket.isClosed();

    }

    /**
     * Завершение работы клиентского потока.
     *
     */
    public synchronized void close() {

        if (closed) {
            MainLogger.info(TAG() + "client already closed.");
            return;
        } // if

        // Завершение работы service.

        MainLogger.info(TAG() + "shutdown client executor.");
        service.shutdownNow();

        // Закрытие потоков.

        MainLogger.info(TAG() + "close local " +
            "& secure streams.");

        if (localSocketInputStream != null) {

            try {
                localSocketInputStream.close();
            } catch (Exception e) {
                // ignore
            }

        } // if

        if (localSocketOutputStream != null) {

            try {
                localSocketOutputStream.close();
            } catch (Exception e) {
                // ignore
            }

        } // if

        if (remoteSocketInputStream != null) {

            try {
                remoteSocketInputStream.close();
            } catch (Exception e) {
                // ignore
            }

        } // if

        if (remoteSocketOutputStream != null) {

            try {
                remoteSocketOutputStream.close();
            } catch (Exception e) {
                // ignore
            }

        } // if

        // Завершение работы сокетов.

        try {
            socket.close();
        } catch (IOException e) {
            // ignore
        }

        try {
            secureSocket.close();
        } catch (IOException e) {
            // ignore
        }

        if (onConnectionListener != null) {
            onConnectionListener.onClientStop(clientId);
        } // if

        closed = true;

    }

    /**
     * Тэг лога.
     *
     * @return тэг.
     */
    private String TAG() {
        return "[" + getName() + "] :: [client-" + clientId + "] ";
    }

}
