/**
 * Copyright 2004-2012 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package userSamples.ca15;

import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCPRequest.ca15.request.CA15GostCertificateRequest;
import ru.CryptoPro.JCPRequest.ca15.status.CA15RequestStatus;
import ru.CryptoPro.JCPRequest.ca15.status.CA15Status;
import ru.CryptoPro.JCPRequest.ca15.user.CA15User;

import java.io.ByteArrayInputStream;
import java.security.*;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

/**
 * Пример создания запроса на сертификат, отправки его в УЦ (CA15)
 * для выпуска сертификата, проверки статуса сертификата и  его
 * обработки.
 *
 * 10/11/2012
 *
 */
public class SendRequestAndGetCertificateExample extends CAConfiguration {

    /**
     * Конструктор.
     *
     * @param needInit Флаг необходимости выполнить
     * инициализацию.
     */
    protected SendRequestAndGetCertificateExample(boolean needInit) {
        super(needInit);
    }

    /**
     *
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {

        final SendRequestAndGetCertificateExample example = new SendRequestAndGetCertificateExample(true);
        example.main();

    }

    /**
     * Функция вывода информации о сертификате.
     *
     * @param cert Сертификат.
     */
    public static void printCertificateInfo(X509Certificate cert) {

        System.out.print("Certificate:");
        System.out.print("\n\tserial number: " + cert.getSerialNumber().toString(16));
        System.out.print("\n\tsubject: " + cert.getSubjectDN());
        System.out.print("\n\tissuer: " + cert.getIssuerDN());
        System.out.println();
    }

    @Override
    public void main() throws Exception {

        // Поля, которые использовались при регистрации пользователя
        // (см. пример RegisterUserExample), необходимо использовать и
        // при генерации запроса. Т.к. обязательным является только поле
        // CommonName, то используем его при генерации запроса на сертификат,
        // а также указывалась страна.
        // Используем пользователя, у которого CommonName и CountryName
        // соответствуют данным (см. пример RegisterUserExample).
        CA15User userInfo = new CA15User(TOKEN_ID, PASSWORD);

        // 1. Генерирование ключевой пары.

        KeyPairGenerator kg = KeyPairGenerator.getInstance(JCP.GOST_EL_DH_NAME);
        KeyPair pair = kg.generateKeyPair();

        // 2. Генерирование запроса на сертификат.

        CA15GostCertificateRequest req = new CA15GostCertificateRequest();
        req.init(JCP.GOST_EL_DH_NAME, false);
        req.setPublicKeyInfo(pair.getPublic());
        req.setSubjectInfo("CN=" + COMMON_NAME + ",C=" + COUNTRY_NAME);
        req.encodeAndSign(pair.getPrivate());

        // 3. Отправляем запрос и получаем статус обработки.

        CA15RequestStatus requestStatus =
            req.sendCertificateRequest(CA15GostCertificateRequest.TEST_CA15_HTTPS_ADDRESS, userInfo);

        if (requestStatus.getValue() == CA15Status.CR_DISP_ERROR) {
            System.err.println("Send certificate request failure.");
            return;
        } // if

        System.out.println("Request identifier: " + requestStatus.getRequestIdentifier()
            + ", status: " + requestStatus.toString());

        // 4. Проверим статус сертификата специально еще раз.

        requestStatus = CA15GostCertificateRequest.checkCertificateStatus(
            CA15GostCertificateRequest.TEST_CA15_HTTPS_ADDRESS, userInfo,
            requestStatus.getRequestIdentifier());

        System.out.println("Request identifier (again): " + requestStatus.getRequestIdentifier()
            + ", status: " + requestStatus.toString());

        if (requestStatus.getValue() == CA15Status.CR_DISP_ERROR) {
            System.err.println("Certificate request has invalid status.");
            return;
        } // if

        byte[] certificateEncoded = null;

        switch (requestStatus.getValue()) {

            // 5. Запрос обработан, просто получаем сертификат.

            case CA15Status.CR_DISP_ISSUED : {
                certificateEncoded =
                    CA15GostCertificateRequest.getCertificateByRequestId(
                        CA15GostCertificateRequest.TEST_CA15_HTTPS_ADDRESS, userInfo,
                            requestStatus.getRequestIdentifier());
            }
            break;

            // 6. Ждем, когда будет обработан запрос и создан сертификат, затем получаем
            // сертификат. Можно организовать процесс ожидания иначе: с помощью цикла опроса
            // состояния обработки и др.

            case CA15Status.CR_DISP_UNDER_SUBMISSION : {

                Thread.sleep(30 * 1000);

                certificateEncoded =
                    CA15GostCertificateRequest.getCertificateByRequestId(
                        CA15GostCertificateRequest.TEST_CA15_HTTPS_ADDRESS, userInfo,
                            requestStatus.getRequestIdentifier());
            }
            break;

            default: {
                System.err.println("Certificate request has invalid status");
                return;
            }

        } // switch

        // 7. Создаем сертификат и выводим на экран.

        if (certificateEncoded != null) {

            X509Certificate cert =
                (X509Certificate) CertificateFactory.getInstance("X.509")
                    .generateCertificate(new ByteArrayInputStream(certificateEncoded));

            printCertificateInfo(cert);

        } // if
    }

}
